#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mcproto_1 = require("mcproto");
let port = 25565;
const servers = new Map;
let isPortOpt = false;
for (const arg of process.argv.slice(2)) {
    if (arg.startsWith("-p"))
        isPortOpt = true;
    else if (isPortOpt) {
        isPortOpt = false;
        port = parseInt(arg);
    }
    else {
        const [local, remote] = arg.split("=");
        const [host, port] = remote.split(":");
        if (!local)
            continue;
        servers.set(local.trim(), {
            host: host.trim() || "127.0.0.1",
            port: parseInt(port) || 25565
        });
    }
}
new mcproto_1.Server((client) => __awaiter(void 0, void 0, void 0, function* () {
    const remoteAddr = client.socket.remoteAddress.replace("::ffff:", "");
    const handshake = yield client.nextPacket();
    const protocol = handshake.readVarInt();
    const address = handshake.readString().split("\x00")[0];
    const log = (code, text = "") => {
        const isoDate = new Date().toISOString();
        if (text)
            text = " - " + text;
        console.log(`${isoDate} ${code} ${remoteAddr} ${address}` + text);
    };
    const serverAddr = servers.get(address);
    if (!serverAddr) {
        const msg = { text: "Please use a valid address to connect!", color: "red" };
        if (client.state == mcproto_1.State.Status) {
            client.on("packet", packet => {
                if (packet.id == 0x0)
                    client.send(new mcproto_1.PacketWriter(0x0).writeJSON({
                        version: { name: "Proxy", protocol: -1 },
                        players: { max: -1, online: -1 },
                        description: msg
                    }));
                if (packet.id == 0x1)
                    client.send(new mcproto_1.PacketWriter(0x1).write(packet.read(8)));
            });
        }
        else if (client.state == mcproto_1.State.Login) {
            client.end(new mcproto_1.PacketWriter(0x0).writeJSON(msg));
        }
        log("BAD_ADDR");
        return setTimeout(() => client.end(), 1000);
    }
    client.on("error", error => log("ERROR", error));
    client.pause();
    const { host, port } = serverAddr;
    let conn;
    try {
        conn = yield mcproto_1.Client.connect(host, port);
    }
    catch (error) {
        log("ERROR", error);
        return client.end();
    }
    conn.on("error", error => log("ERROR", error));
    log("CONNECT");
    conn.send(new mcproto_1.PacketWriter(0x0).writeVarInt(protocol)
        .writeString(host).writeUInt16(port)
        .writeVarInt(client.state));
    client.on("packet", packet => conn.send(packet));
    client.resume();
    client.unpipe(), conn.unpipe();
    client.socket.pipe(conn.socket, { end: true });
    conn.socket.pipe(client.socket, { end: true });
})).listen(port);
console.log("Server listening on port " + port);
servers.forEach(({ host, port }, localAddr) => {
    console.log(`  - ${localAddr} -> ${host}:${port}`);
});
console.log();
